'==============================================================================
' Copyright  Micromega Corporation 2004-2005. All rights reserved
'
' File: umfpuV2-i2c.bas
'
' This file provides support routines for using the uM-FPU V2 floating point
' coprocessor with the meLabs PicBasic Pro compiler. It supports the I2C
' interface. (A seperate file is available for supported the SPI interface.)
' This file is added to the main program using an include statement.  The user
' must set the definitions for the oscillator speed and interface pins
' appropriately (see examples below).
'
' @author Cam Thompson, Micromega Corporation
' @version V2.0 - January 10, 2005
' changes:
'   V2.0 - January 18, 2005
'	  - modified for PicBasic Pro compiler
'   V1.0 - April 29, 2004
'     - original version
'==============================================================================

'-------------------- uM-FPU V2 definitions -----------------------------------

'jump to end of code (in case this file is included at beginning of the program)
  goto FPU_end   

'-------------------- PIC oscillator speed ------------------------------------
' Note: The oscillator speed must be defined to ensure proper timing constants
' for the uM-FPU routines. These definitions are normally included in the main
' program. An example of the required definitions is as follows:
'
'   define          OSC     20              'specify the speed of the oscillator
'   OSC_SPEED       con     20              'define PicBasic constant

'-------------------- uM-FPU pin definitions ----------------------------------
' Note: Fpu_SCL and Fpu_SDA must be defined according to the pin assignments
' used in your application. These definitions are normally made in the main
' program. The I2C pins require 4.7K pullup resistors. Refer to the PicBasic
' Pro manual for documentation for the I2CREAD and I2CWRITE commands. Fpu_ID
' specifies the device address for the uM-FPU.  The default address is $C8.
' I2C_HOLD must be defined for proper I2C timing.
'
'   Fpu_SCL         var     PORTC.3         'I2C clock pin
'   Fpu_SDA         var     PORTC.4         'I2C data pin
'   Fpu_ID          con     $C8             'uM-FPU I2C device address
'   define          I2C_HOLD 1              'required for correct I2C timing

'-------------------- uM-FPU constants ----------------------------------------

ReadDelay       con     180             'delay before first read byte

'-------------------- uM-FPU opcodes ------------------------------------------

SELECTA         con     $00             'select A register
SELECTB         con     $10             'select B register
WRITEA          con     $20             'select A and write float to register
WRITEB          con     $30             'Select B and write float to register
FREAD           con     $40             'read float from register
FSET            con     $50             'A = REG
LSET            con     $50             'A = REG

FADD            con     $60             'A = A + REG (float)
FSUB            con     $70             'A = A - REG (float)
FMUL            con     $80             'A = A * REG (float)
FDIV            con     $90             'A = A / REG (float)

LADD            con     $A0             'A = A + REG (long)
LSUB            con     $B0             'A = A - REG (long)
LMUL            con     $C0             'A = A * REG (long)
LDIV            con     $D0             'A = A / REG (long)

SQRT            con     $E0             'A = sqrt(A)
LOG             con     $E1             'A = ln(A)
LOG10           con     $E2             'A = log(A)
EXP             con     $E3             'A = e ** A
EXP10           con     $E4             'A = 10 ** A
FSIN            con     $E5             'A = sin(A) radians
FCOS            con     $E6             'A = cos(A) radians
FTAN            con     $E7             'A = tan(A) radians
FLOOR           con     $E8             'A = nearest integer <= A
CEIL            con     $E9             'A = nearest integer >= A
ROUND           con     $EA             'A = nearest integer to A
NEGATE          con     $EB             'A = -A
FABS            con     $EC             'A = |A|
INVERSE         con     $ED             'A = 1 / A
DEGREES         con     $EE             'A = A / (PI / 180) radians to degrees
RADIANS         con     $EF             'A = A * (PI / 180) degreees to radians
SYNC            con     $F0             'synchronization
FLOAT           con     $F1             'copy A to register 0 and float
FIX             con     $F2             'copy A to register 0 and fix
FCOMPARE        con     $F3             'compare A and B
LOADBYTE        con     $F4             'write signed byte to reg0, convert to float
LOADUBYTE       con     $F5             'write unsigned byte to reg0, convert to float
LOADWORD        con     $F6             'write signed word to reg0, convert to float
LOADUWORD       con     $F7             'write unsigned word to reg0, convert to float
READSTR         con     $F8             'read zero terminated string
ATOF            con     $F9             'convert ASCII to float, store in A
FTOA            con     $FA             'convert float to ASCII
ATOL            con     $FB             'convert ASCII to long, store in A
LTOA            con     $FC             'convert long to ASCII
FSTATUS         con     $FD             'get the status of A register
XOP             con     $FE             'extended opcode
NOP             con     $FF             'nop

FUNCTION        con     $00             '(XOP) user functions 0-15
READBYTE        con     $90             '(XOP) read low 8 bits of A (long)
READWORD        con     $91             '(XOP) read low 16 bits of A (long)
READLONG        con     $92             '(XOP) read 32-bit long value from A
READFLOAT       con     $93             '(XOP) read floating point value from A
LINCA           con     $94             '(XOP) A = A + 1 (long)
LINCB           con     $95             '(XOP) A = A + 1 (long)
LDECA           con     $96             '(XOP) A = A - 1 (long)
LDECB           con     $97             '(XOP) A = A - 1 (long)
LAND            con     $98             '(XOP) A = A AND B (long)
LOR             con     $99             '(XOP) A = A OR B (long)
LXOR            con     $9A             '(XOP) A = A XOR B (long)
LNOT            con     $9B             '(XOP) A = NOT A (long)
LTST            con     $9C             '(XOP) status of A AND B (long)
LSHIFT          con     $9D             '(XOP) shift A by B bits (long)
LWRITEA         con     $A0             '(XOP) select A and write long to register
LWRITEB         con     $B0             '(XOP) select B and write long to register
LREAD           con     $C0             '(XOP) read 32-bit long from register
LUDIV           con     $D0             '(XOP) A = A / REG (unsigned long)
POWER           con     $E0             '(XOP) A = A ** B
ROOT            con     $E1             '(XOP) A = the Bth root of A
FMIN            con     $E2             '(XOP) A = minimum of A and B
FMAX            con     $E3             '(XOP) A = maximum of A and B
FRACTION        con     $E4             '(XOP) load reg0 with the fractional part of A
ASIN            con     $E5             '(XOP) A = asin(A) radians
ACOS            con     $E6             '(XOP) A = acos(A) radians
ATAN            con     $E7             '(XOP) A = atan(A) radians
ATAN2           con     $E8             '(XOP) A = atan(A/B)
LCOMPARE        con     $E9             '(XOP) long compare A and B
LUCOMPARE       con     $EA             '(XOP) unsigned long compare A and B
LSTATUS         con     $EB             '(XOP) long status
LNEGATE         con     $EC             '(XOP) A = -A (long)
LABS            con     $ED             '(XOP) A = |A| (long)
LEFT            con     $EE             '(XOP) right parenthesis
RIGHT           con     $EF             '(XOP) left parenthesis
LOADZERO        con     $F0             '(XOP) load register 0 with zero
LOADONE         con     $F1             '(XOP) load register 0 with 1.0
LOADE           con     $F2             '(XOP) load register 0 with e
LOADPI          con     $F3             '(XOP) load register 0 with pi
LONGBYTE        con     $F4             '(XOP) write signed byte to reg0, convert to long
LONGUBYTE       con     $F5             '(XOP) write unsigned byte to reg0, convert to long
LONGWORD        con     $F6             '(XOP) write signed word to reg0, convert to long
LONGUWORD       con     $F7             '(XOP) write unsigned word to reg0, convert to long
IEEEMODE        con     $F8             '(XOP) set IEEE mode (default)
PICMODE         con     $F9             '(XOP) set PIC mode
CHECKSUM        con     $FA             '(XOP) calculate code checksum
BREAK           con     $FB             '(XOP) debug breakpoint
TRACEOFF        con     $FC             '(XOP) turn debug trace off
TRACEON         con     $FD             '(XOP) turn debug trace on
TRACESTR        con     $FE             '(XOP) send debug string to trace buffer
VERSION         con     $FF             '(XOP) get version string

SyncChar        con     $5C             'sync character

'-------------------- uM-FPU variables ----------------------------------------

dataWord        var     word            'data word
dataHigh        var     dataWord.HIGHBYTE 'high byte of dataWord
dataLow         var     dataword.LOWBYTE  'low byte of dataLow
dataByte        var     dataLow         '(alternate name)
opcode          var     dataHigh        'opcode (same as dataHigh)
reg             var     dataHigh        'register (same as dataHigh)
format          var     dataLow         'format (same as dataLow)
fpu_status      var     dataLow         'status (same as dataLow)

status_Zero     var     fpu_status.0    'Zero status bit (0-not zero, 1-zero)
status_Sign     var     fpu_status.1    'Sign status bit (0-positive, 1-negative)
status_NaN      var     fpu_status.2    'Not a Number status bit (0-valid number, 1-NaN)
status_Inf      var     fpu_status.3    'Infinity status bit (0-not infinite, 1-infinite)

'==================== end of uM-FPU definitions ==============================

'==============================================================================
'-------------------- uM-FPU SPI support routines (V2.0) ----------------------
'==============================================================================

Fpu_Reset:                              'send reset command to uM-FPU
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 1, [0]
  PAUSE 8
                                        'check for synchronization
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [SYNC]
                                        '(fall through to Fpu_ReadStatus)
Fpu_ReadStatus:
  PAUSEUS ReadDelay                     'wait for read setup delay
  I2CREAD Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [fpu_status] 'read the status
  RETURN

Fpu_Wait:                               'wait for ready status
  I2CREAD Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [fpu_status]
  IF fpu_status <> 0 THEN GOTO Fpu_wait
  RETURN

Fpu_ReadDelay:
  PAUSEUS ReadDelay                     'delay for read setup
  RETURN

Print_Version:                          'get the uM-FPU version string
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [XOP, VERSION]
  GOTO Print_String2                    'print it

Print_Float:                            'set format to zero (free format)
  format = 0                            '(fall through to Print_FloatFormat)

Print_FloatFormat:
  opcode = FTOA                         'convert floating point to formatted ASCII
  GOTO Print_String                     'print the value

Print_Long:                             'set format to 0 (free format)
  format = 0                            '(fall through to Print_LongFormat)

Print_LongFormat:                       'convert long integer to formatted ASCII
  opcode = LTOA                         '(fall through to Print_String)

Print_String:                           'send conversion command
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [opcode, format]

Print_String2:
  GOSUB Fpu_Wait                        'wait until uM-FPU is ready
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [READSTR]
  PAUSEUS ReadDelay

Print_String3:                          'display zero terminated string
  I2CREAD Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [dataByte]
  IF (dataByte = 0 OR dataByte > 127) THEN RETURN
  DEBUG dataByte
  GOTO Print_String3

FPU_end:                                'end of umfpuV2-i2c.bas

